unit ulistaplantillassr3;

{$mode delphi}
(*
Objetivo: Definición de la clase TListaPlantillas_SR3 como una "cosa con nombre"
que sirve para agregar a la Sala el listado de plantillas.

Esta clase también debe ser capaz de colaborar en el empaquetado de la sala
y también en la formación del monitor a ejecutar en la sala en base a las
Plantillas ACTIVAS.

En el editor debe estar la posibilidad de Activar y/o Desactivar las plantillas.

*)
interface

uses
  Classes, SysUtils, ucosa, ucosaconnombre;

type

  { TPlantillaSimRes3_rec }

  TPlantillaSimRes3_rec = class(TCosaConNombre)
    activa: boolean;
    archi: TArchiRef;
    constructor Create(capa: integer; nombre, xarchi: string);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;

  end;

  { TListaPlantillasSimRes3 }

  TListaPlantillasSimRes3 = class(TListaDeCosasConNombre)
    next_nid: integer;
    constructor Create(capa: integer; nombre: string);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    // chequea si el archivo ya no está en la lista y lo agrega.
    // retorna el puntero al Rec agregado o NIL si ya estaba y no fue agregado.
    function AppendArchivo(nombreArchivo: string): TPlantillaSimRes3_rec;

    // retorna NIL si no encuentra una ficha que apunte al archivo nombreArchivo
    function FindArchivo(nombreArchivo: string): TPlantillaSimRes3_rec;

    function lista_activas: TStrings;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;

  end;


procedure AlInicio;
procedure AlFinal;

implementation


constructor TPlantillaSimRes3_rec.Create(capa: integer; nombre, xarchi: string);
begin
  inherited Create(capa, nombre);
  activa := True;
  archi := TArchiRef.Create(xarchi);
end;

constructor TPlantillaSimRes3_rec.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rd('activa', activa);
  f.rdArchRef('archi', archi);
end;

procedure TPlantillaSimRes3_rec.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('activa', activa);
  f.wrArchRef('archi', archi);
end;

class function TPlantillaSimRes3_rec.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TPlantillaSimRes3_rec, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('activa',TPlantillaSimRes3_rec(nil).activa, TDataTypeBooleanOfCosa, '', 113, -1);
  list.AddDataColumn('archi', TPlantillaSimRes3_rec(nil).archi, TDataTypeArchiRefOfCosa, '', 113, -1);
  Result:=list;
end;

constructor TListaPlantillasSimRes3.Create(capa: integer; nombre: string);
begin
  inherited Create(capa, nombre);
  next_nid := 1;
end;

constructor TListaPlantillasSimRes3.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rd('next_nid', next_nid);
end;

procedure TListaPlantillasSimRes3.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('next_nid', next_nid);
end;


function TListaPlantillasSimRes3.FindArchivo(nombreArchivo: string):
TPlantillaSimRes3_rec;
var
  rec: TPlantillaSimRes3_rec;
  k: integer;
  buscando: boolean;
begin
  buscando := True;
  k := 0;
  while buscando and (k < Count) do
  begin
    rec := items[k] as TPlantillaSimRes3_rec;
    if (rec.archi.archi = nombreArchivo) then
      buscando := False
    else
      Inc(k);
  end;
  if buscando then
    Result := nil
  else
    Result := rec;
end;


function TListaPlantillasSimRes3.lista_activas: TStrings;

var
  k: integer;
  rec: TPlantillaSimRes3_rec;
  res: TStringList;

begin
  res := TStringList.Create;
  for k := 0 to Count - 1 do
  begin
    rec := items[k] as TPlantillaSimRes3_rec;
    if rec.activa then
      res.Add(rec.archi.archi);
  end;
  Result := res;
end;

function TListaPlantillasSimRes3.AppendArchivo(nombreArchivo: string):
TPlantillaSimRes3_rec;
var
  rec: TPlantillaSimRes3_rec;
begin
  if FindArchivo(nombreArchivo) = nil then
  begin
    rec := TPlantillaSimRes3_rec.Create(0, 'rec_' + IntToStr(next_nid), nombreArchivo);
    Inc(next_nid);
    add(rec);
    Result := rec;
  end
  else
    Result := nil;
end;

class function TListaPlantillasSimRes3.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
 list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TListaPlantillasSimRes3, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('next_nid',TListaPlantillasSimRes3(nil).next_nid, TDataTypeIntegerOfCosa, '', 113, -1);
  Result:=list;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TPlantillaSimRes3_rec.ClassName,
    TPlantillaSimRes3_rec);
  registrarClaseDeCosa(TListaPlantillasSimRes3.ClassName,
    TListaPlantillasSimRes3);
end;

procedure AlFinal;
begin
end;


end.
